SystemOrganization addCategory: #'Swazoo-HTTP-Headers'!
SystemOrganization addCategory: #'Swazoo-HTTP-Messages'!
SystemOrganization addCategory: #'Swazoo-HTTP-Support'!
SystemOrganization addCategory: #'Swazoo-HTTP-Tests'!

TestCase subclass: #HTTPRequestTest
	instanceVariableNames: 'request'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Swazoo-HTTP-Tests'!

!HTTPRequestTest methodsFor: 'private' stamp: ''!
basicGet
	| ws |
	ws := WriteStream on: String new.
	ws nextPutAll: 'GET / HTTP/1.1'.
	self crlfOn: ws.
	ws nextPutAll: 'Host: foo.com'.
	self crlfOn: ws.
	self crlfOn: ws.
	^HTTPRequest readFrom: (ReadStream on: ws contents)! !

!HTTPRequestTest methodsFor: 'private' stamp: ''!
basicGetHTTP10
	| ws |
	ws := WriteStream on: String new.
	ws nextPutAll: 'GET / HTTP/1.0'.
	self crlfOn: ws.
	self crlfOn: ws.
	^HTTPRequest readFrom: (ReadStream on: ws contents)! !

!HTTPRequestTest methodsFor: 'private' stamp: ''!
basicGetHTTP10Keepalive
	| ws |
	ws := WriteStream on: String new.
	ws nextPutAll: 'GET / HTTP/1.0'.
	self crlfOn: ws.
	ws nextPutAll: 'Connection: Keep-Alive'.
	self crlfOn: ws.
	self crlfOn: ws.
	^HTTPRequest readFrom: (ReadStream on: ws contents)! !

!HTTPRequestTest methodsFor: 'private' stamp: ''!
basicHead
	| ws |
	ws := WriteStream on: String new.
	ws nextPutAll: 'HEAD / HTTP/1.1'.
	self crlfOn: ws.
	ws nextPutAll: 'Host: foo.com'.
	self crlfOn: ws.
	self crlfOn: ws.
	^HTTPRequest readFrom: (ReadStream on: ws contents)! !

!HTTPRequestTest methodsFor: 'private' stamp: ''!
crlfOn: aStream 
	aStream
		nextPut: Character cr;
		nextPut: Character lf! !

!HTTPRequestTest methodsFor: 'private' stamp: ''!
fullGet
	| ws |
	ws := WriteStream on: String new.
	ws nextPutAll: 'GET /aaa/bbb/ccc.html?foo=bar&baz=quux HTTP/1.1'.
	self crlfOn: ws.
	ws nextPutAll: 'Connection: Keep-Alive'.
	self crlfOn: ws.
	ws nextPutAll: 'User-Agent: Mozilla/4.72 [en] (X11; I; Linux 2.3.51 i686)'.
	self crlfOn: ws.
	ws nextPutAll: 'Host: foo.com:8888'.
	self crlfOn: ws.
	ws nextPutAll: 'Referer: http://www.bar.com/takeMeThere.html'.
	self crlfOn: ws.
	self crlfOn: ws.
	^HTTPRequest readFrom: (ReadStream on: ws contents)! !

!HTTPRequestTest methodsFor: 'private' stamp: ''!
portedGet
	| ws |
	ws := WriteStream on: String new.
	ws nextPutAll: 'GET / HTTP/1.1'.
	self crlfOn: ws.
	ws nextPutAll: 'Host: foo.com:8888'.
	self crlfOn: ws.
	self crlfOn: ws.
	^HTTPRequest readFrom: (ReadStream on: ws contents)! !

!HTTPRequestTest methodsFor: 'testing' stamp: ''!
test10ConnectionClose
	request := self basicGetHTTP10.
	self assert: request wantsConnectionClose! !

!HTTPRequestTest methodsFor: 'testing' stamp: ''!
test10KeepAliveConnectionClose
	request := self basicGetHTTP10Keepalive.
	self deny: request wantsConnectionClose! !

!HTTPRequestTest methodsFor: 'testing' stamp: ''!
testBasicGet
	request := self basicGet.
	self assert: request isGet.
	self deny: request isHead.
	self deny: request isPost.
	self deny: request isPut! !

!HTTPRequestTest methodsFor: 'testing' stamp: ''!
testBasicGetHost
	request := self basicGet.
	self assert: request uri hostname = 'foo.com'! !

!HTTPRequestTest methodsFor: 'testing' stamp: ''!
testBasicGetPort
	request := self basicGet.
	self assert: request port = 80! !

!HTTPRequestTest methodsFor: 'testing' stamp: ''!
testBasicHead
	request := self basicHead.
	self assert: request isHead.
	self deny: request isGet.
	self deny: request isPost.
	self deny: request isPut! !

!HTTPRequestTest methodsFor: 'testing' stamp: ''!
testConnection
	request := self fullGet.
	self assert: request connection = 'Keep-Alive'! !

!HTTPRequestTest methodsFor: 'testing' stamp: ''!
testHeaderAtIfPresent
	| result |
	request := self basicGet.
	result := request headerAt: 'NO-SUCH-HEADER'
				ifPresent: [:header | self signalFailure: 'Should not be executed!!'].
	self assert: result isNil.
	result := request headerAt: 'HOST'
				ifPresent: 
					[:header | 
					self assert: header == (request headerAt: 'HOST').
					true].
	self assert: result! !

!HTTPRequestTest methodsFor: 'testing' stamp: ''!
testNo11ConnectionClose
	request := self basicGet.
	self deny: request wantsConnectionClose! !

!HTTPRequestTest methodsFor: 'testing' stamp: ''!
testNoEqualsQueries
	| ws |

	ws := WriteStream on: String new.
	ws nextPutAll: 'GET /test/typed.asmx?WSDL HTTP/1.1'.
	self crlfOn: ws.
	ws nextPutAll: 'Host: foo.com:8888'.
	self crlfOn: ws.
	self crlfOn: ws.
	request := HTTPRequest readFrom: (ReadStream on: ws contents).

	self assert: (request includesQuery: 'WSDL').
	self assert: (request queryAt: 'WSDL') isNil.
	"self assert: (request uri queryString = 'WSDL')."! !

!HTTPRequestTest methodsFor: 'testing' stamp: ''!
testPortedGetPort
	request := self portedGet.
	self assert: request port = 8888! !

!HTTPRequestTest methodsFor: 'testing' stamp: 'avi 4/16/2004 16:15'!
testPostRawEntity
	| ws post|
	ws := WriteStream on: String new.
	ws nextPutAll: 'POST /foobar HTTP/1.0'.
	self crlfOn: ws.
	ws nextPutAll: 'Host: foo.com'.
	self crlfOn: ws.
	ws nextPutAll: 'Content-Type: text/plain'.
	self crlfOn: ws.
	ws nextPutAll: 'Content-Length: 12'.
	self crlfOn: ws.
	self crlfOn: ws.
	ws nextPutAll:  'Hello, World'.
	post := HTTPRequest readFrom: (ReadStream on: ws contents).
	self assert: post postData isEmpty.
	self assert: post entityBody = 'Hello, World'.! !

!HTTPRequestTest methodsFor: 'testing' stamp: 'avi 4/16/2004 16:14'!
testPostUrlEncodedData
	| ws post|
	ws := WriteStream on: String new.
	ws nextPutAll: 'POST / HTTP/1.1'.
	self crlfOn: ws.
	ws nextPutAll: 'Host: foo.com'.
	self crlfOn: ws.
	ws nextPutAll: 'Content-Type: application/x-www-form-urlencoded'.
	self crlfOn: ws.
	ws nextPutAll: 'Content-Length: 31'.
	self crlfOn: ws.
	self crlfOn: ws.
	ws nextPutAll: 'address=+fs&product=&quantity=1'.
	post := HTTPRequest readFrom: (ReadStream on: ws contents).
	self assert: (post postDataAt: 'address') value = ' fs'.
	self assert: (post postDataAt: 'product') value = ''.
	self assert: (post postDataAt: 'quantity') value = '1'.! !

!HTTPRequestTest methodsFor: 'testing' stamp: ''!
testReferer
	request := self fullGet.
	self assert: request referer = 'http://www.bar.com/takeMeThere.html'! !

!HTTPRequestTest methodsFor: 'testing' stamp: ''!
testUserAgent
	request := self fullGet.
	self 
		assert: request userAgent = 'Mozilla/4.72 [en] (X11; I; Linux 2.3.51 i686)'! !

TestCase subclass: #HTTPResponseTest
	instanceVariableNames: 'response'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Swazoo-HTTP-Tests'!

!HTTPResponseTest methodsFor: 'private' stamp: ''!
crlf
	^String with: Character cr with: Character lf! !

!HTTPResponseTest methodsFor: 'testing' stamp: ''!
testInternalServerError
	| ws ls |
	response := HTTPResponse internalServerError.
	ws := WriteStream on: String new.
	response printStatusOn: ws.
	ls := LineStream on: (ReadStream on: ws contents).
	self assert: ls nextLine = 'HTTP/1.1 500 Internal Server Error'! !

!HTTPResponseTest methodsFor: 'testing' stamp: ''!
testOK
	| ws ls |
	response := HTTPResponse ok.
	ws := WriteStream on: String new.
	response printStatusOn: ws.
	ls := LineStream on: (ReadStream on: ws contents).
	self assert: ls nextLine = 'HTTP/1.1 200 OK'! !

TestCase subclass: #LineStreamTest
	instanceVariableNames: 'stream'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Swazoo-HTTP-Tests'!

!LineStreamTest methodsFor: 'private' stamp: ''!
crlfOn: ws 
	ws
		nextPut: Character cr;
		nextPut: Character lf! !

!LineStreamTest methodsFor: 'testing' stamp: ''!
testLinesWithDoubleCRLF
	| ws comparisonString |
	comparisonString := 'abcd'.
	ws := WriteStream on: String new.
	ws nextPutAll: comparisonString.
	self crlfOn: ws.
	self crlfOn: ws.
	stream := LineStream on: (ReadStream on: ws contents).
	self assert: stream nextLine = comparisonString.
	self assert: stream nextLine = ''! !

!LineStreamTest methodsFor: 'testing' stamp: ''!
testSingleLineWithCR
	| ws comparisonString errored |
	comparisonString := 'abcd' , (String with: Character cr) , 'efg'.
	ws := WriteStream on: String new.
	ws nextPutAll: comparisonString.
	ws nextPut: Character cr.
	stream := LineStream on: (ReadStream on: ws contents).
	errored := false.
	[stream nextLine] on: Error do: [:ex | errored := true].
	self assert: errored! !

!LineStreamTest methodsFor: 'testing' stamp: ''!
testSingleLineWithCRLF
	| ws comparisonString |
	comparisonString := 'abcd'.
	ws := WriteStream on: String new.
	ws nextPutAll: comparisonString.
	self crlfOn: ws.
	stream := LineStream on: (ReadStream on: ws contents).
	self assert: stream nextLine = comparisonString! !

TestCase subclass: #SwazooURITest
	instanceVariableNames: 'fooURI barURI queryURI'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Swazoo-HTTP-Tests'!

!SwazooURITest methodsFor: 'running' stamp: ''!
setUp
	fooURI := (SwazooURI new)
				hostname: 'www.foo.com';
				identifier: '/index.html'.
	queryURI := (SwazooURI new)
				hostname: 'www.foo.com';
				identifier: '/index.html';
				queries: 'foo=1&bar=hi'.
	barURI := (SwazooURI new)
				hostname: 'www.bar.com';
				identifier: '/files/';
				port: 8080! !

!SwazooURITest methodsFor: 'running' stamp: ''!
testHostname
	self assert: fooURI hostname = 'www.foo.com'.
	self assert: queryURI hostname = 'www.foo.com'.
	self assert: barURI hostname = 'www.bar.com'! !

!SwazooURITest methodsFor: 'running' stamp: ''!
testIdentifier
	self assert: fooURI identifier = '/index.html'.
	self assert: queryURI identifier = '/index.html'.
	self assert: barURI identifier = '/files/'! !

!SwazooURITest methodsFor: 'running' stamp: ''!
testIdentifierPath
	self assert: fooURI identifierPath 
				= (OrderedCollection with: '/' with: 'index.html').
	self assert: queryURI identifierPath 
				= (OrderedCollection with: '/' with: 'index.html').
	self 
		assert: barURI identifierPath = (OrderedCollection with: '/' with: 'files')! !

!SwazooURITest methodsFor: 'running' stamp: ''!
testIsDirectory
	self deny: fooURI isDirectory.
	self deny: queryURI isDirectory.
	self assert: barURI isDirectory! !

!SwazooURITest methodsFor: 'running' stamp: ''!
testPort
	self assert: fooURI port = 80.
	self assert: queryURI port = 80.
	self assert: barURI port = 8080! !

!SwazooURITest methodsFor: 'running' stamp: ''!
testQueries
	self deny: (queryURI includesQuery: 'hi').
	self assert: (queryURI includesQuery: 'foo').
	self assert: (queryURI includesQuery: 'bar').
	self assert: (queryURI queryAt: 'foo') = '1'.
	self assert: (queryURI queryAt: 'bar') = 'hi'! !

!SwazooURITest methodsFor: 'running' stamp: ''!
testValue
	self assert: fooURI value = 'http://www.foo.com/index.html'.
	self assert: queryURI value = 'http://www.foo.com/index.html?foo=1&bar=hi'.
	self assert: barURI value = 'http://www.bar.com:8080/files/'! !

!Character methodsFor: '*swazoo-http' stamp: ''!
isHTTPReserved
	^(self isAlphaNumeric or: ['-_.!!~*''()' includes: self]) not! !

!SocketStream methodsFor: '*swazoo-http' stamp: 'avi 4/16/2004 16:53'!
nextPutBytes: aByteArray
	self nextPutAll: aByteArray asString! !

!SocketStream methodsFor: '*swazoo-http' stamp: 'avi 4/16/2004 16:51'!
peek
	self atEnd ifTrue: [^false].
	self inStream atEnd ifTrue: 
		[self receiveData.
		self atEnd ifTrue: [^false]].
	^self inStream peek! !

!DateAndTime methodsFor: '*swazoo-http' stamp: 'avi 4/16/2004 16:25'!
rfc1123String
	"totally wrong format"
	#FIXME.
	
	^ self asDate asString, ' ', self asTime asString! !

!PositionableStream methodsFor: '*swazoo-http' stamp: 'ab 8/19/2003 13:23'!
skipThrough: aCharacter
	^ self upTo: aCharacter! !

!SequenceableCollection methodsFor: '*swazoo-http' stamp: ''!
tokensBasedOnAll: aSequenceableCollection 
	"Answer an OrderedCollection of the sub-sequences
	 of the receiver that are separated by aSequenceableCollection."

	| i selfPos oc |
	selfPos := 1.
	oc := OrderedCollection new.
	
	[i := self indexOfSubCollection: aSequenceableCollection startingAt: selfPos.
	i = 0] 
			whileFalse: 
				[oc add: (self copyFrom: selfPos to: i - 1).
				selfPos := i + aSequenceableCollection size].
	oc add: (self copyFrom: selfPos to: self size).
	^oc! !

Object subclass: #HTTPMessage
	instanceVariableNames: 'headers'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Swazoo-HTTP-Messages'!

!HTTPMessage class methodsFor: 'instance creation' stamp: ''!
new
	^super new initialize! !

!HTTPMessage methodsFor: 'accessing-headers' stamp: ''!
addHeader: aHeader 
	"HTTPSpec1.1 Sec4.2
Multiple message-header fields with the same field-name MAY be present in a message if and only if the entire field-value for that header field is defined as a comma-separated list [i.e., #(values)]. It MUST be possible to combine the multiple header fields into one 'field-name: field-value' pair, without changing the semantics of the message, by appending each subsequent field-value to the first, each separated by a comma. The order in which header fields with the same field-name are received is therefore significant to the interpretation of the combined field value, and thus a proxy MUST NOT change the order of these field values when a message is forwarded. "

	(self includesHeader: aHeader name) 
		ifTrue: [(self headerAt: aHeader name) combine: aHeader]
		ifFalse: [self headerAt: aHeader name put: aHeader].
	^aHeader! !

!HTTPMessage methodsFor: 'accessing-headers' stamp: ''!
addHeaderName: aName value: aValue 
	^self addHeader: (GenericHeaderField name: aName value: aValue)! !

!HTTPMessage methodsFor: 'accessing-headers' stamp: ''!
headerAt: aKey 
	^self headerAt: aKey asUppercase ifAbsent: [nil]! !

!HTTPMessage methodsFor: 'accessing-headers' stamp: ''!
headerAt: aKey ifAbsent: aBlock
	^self headers at: aKey asUppercase ifAbsent: aBlock! !

!HTTPMessage methodsFor: 'accessing-headers' stamp: ''!
headerAt: aKey ifPresent: aBlock 
	| header |
	header := self headers at: aKey ifAbsent: [^nil].
	^aBlock value: header! !

!HTTPMessage methodsFor: 'accessing-headers' stamp: ''!
headerAt: aKey put: anObject 
	self headers at: aKey asUppercase put: anObject! !

!HTTPMessage methodsFor: 'accessing-headers' stamp: ''!
headerValueOrNil: aFieldName 
	^(self headerAt: aFieldName ifAbsent: [^nil]) value! !

!HTTPMessage methodsFor: 'accessing-headers' stamp: ''!
headerValuesAt: aString 
	^(self headerAt: aString ifAbsent: [^#()]) values! !

!HTTPMessage methodsFor: 'private-initialize' stamp: ''!
headers
	^headers! !

!HTTPMessage methodsFor: 'accessing-headers' stamp: ''!
includesHeader: aKey 
	^self headers includesKey: aKey asUppercase! !

!HTTPMessage methodsFor: 'private-initialize' stamp: ''!
initHeaders
	headers := Dictionary new! !

!HTTPMessage methodsFor: 'private-initialize' stamp: ''!
initialize
	self initHeaders! !

HTTPMessage subclass: #HTTPRequest
	instanceVariableNames: 'httpVersion peer timestamp ip environmentData uri resolution encrypted authenticated contentsStream'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Swazoo-HTTP-Messages'!

HTTPRequest subclass: #HTTPGet
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Swazoo-HTTP-Messages'!

!HTTPGet methodsFor: 'testing' stamp: ''!
isGet
	^true! !

HTTPRequest subclass: #HTTPHead
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Swazoo-HTTP-Messages'!

!HTTPHead methodsFor: 'testing' stamp: ''!
isHead
	^true! !

HTTPRequest subclass: #HTTPPost
	instanceVariableNames: 'postData entityBody'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Swazoo-HTTP-Messages'!

!HTTPPost methodsFor: 'accessing' stamp: ''!
emptyData
	^self postData select: [:each | each value isEmpty]! !

!HTTPPost methodsFor: 'accessing' stamp: ''!
entityBody
	^entityBody! !

!HTTPPost methodsFor: 'private' stamp: ''!
entityBody: aString
	entityBody := aString! !

!HTTPPost methodsFor: 'private-initialize' stamp: ''!
initPostData
	postData := Dictionary new! !

!HTTPPost methodsFor: 'private-initialize' stamp: ''!
initialize
	super initialize.
	self initPostData! !

!HTTPPost methodsFor: 'testing' stamp: ''!
isPost
	^true! !

!HTTPPost methodsFor: 'private' stamp: ''!
multipartDataFrom: aStream 
	| e tokens boundary |
	(self includesHeader: 'Content-Length') 
		ifFalse: [^self error: 'Bad POST header'].
	e := aStream nextBytes: self contentLength.
	boundary := (self headerAt: 'Content-Type') parameterAt: 'boundary'
				ifAbsent: [^self].
	tokens := e tokensBasedOnAll: boundary asByteArray.
	tokens do: 
			[:bytes | 
			| part |
			part := self partFromBytes: bytes.
			part isNil ifFalse: [self postData add: part]]! !

!HTTPPost methodsFor: 'private' stamp: ''!
partFromBytes: bytes 
	| str rs name datum |
	str := bytes asString trimBlanks.
	str = '--' ifTrue: [^nil].
	rs := str readStream.
	name := nil.
	datum := HTTPPostDatum new.
	[rs atEnd] whileFalse: 
			[| line |
			line := rs upTo: Character cr.
			rs peek = Character lf 
				ifTrue: 
					[| field |
					rs next.
					line isEmpty 
						ifTrue: 
							[| entity |
							name isNil ifTrue: [^nil].
							entity := rs upToEnd.
							entity size < 4 ifTrue: [^self].
							datum value: (entity copyFrom: 1 to: entity size - 4).
							^name -> datum].
					field := HeaderField fromLine: line.
					field isContentDisposition 
						ifTrue: [name := (field parameterAt: 'name') copyWithout: $"].
					field isContentType ifTrue: [datum contentType: field value]]]! !

!HTTPPost methodsFor: 'private' stamp: ''!
postData
	^postData! !

!HTTPPost methodsFor: 'accessing' stamp: ''!
postDataAt: aKey 
	^self postDataAt: aKey ifAbsent: [nil]! !

!HTTPPost methodsFor: 'accessing' stamp: ''!
postDataAt: aKey do: aBlock 
	| val |
	val := self postData at: aKey ifAbsent: [nil].
	val isNil ifFalse: [aBlock value: val]! !

!HTTPPost methodsFor: 'accessing' stamp: ''!
postDataAt: aKey ifAbsent: aBlock 
	^self postData at: aKey ifAbsent: aBlock! !

!HTTPPost methodsFor: 'accessing' stamp: ''!
postDataAt: aKey put: aPostDatum 
	"for testing purposes"

	self postData at: aKey put: aPostDatum! !

!HTTPPost methodsFor: 'accessing' stamp: ''!
postDataAt: aKey putString: aString 
	"for testing purposes"

	self postDataAt: aKey put: (HTTPPostDatum new value: aString)! !

!HTTPPost methodsFor: 'accessing' stamp: ''!
postDataKeys
	^self postData keys! !

!HTTPPost methodsFor: 'accessing' stamp: ''!
postDataStringAt: aKey 
	^(self postDataAt: aKey ifAbsent: [^nil]) value! !

!HTTPPost methodsFor: 'accessing' stamp: ''!
postKeysAndValuesDo: aTwoArgBlock 
	self postData 
		keysAndValuesDo: [:key :each | aTwoArgBlock value: key value: each value]! !

!HTTPPost methodsFor: 'private' stamp: ''!
rawEntityBodyFrom: aStream 
	| crlfBody body |
	crlfBody := (aStream nextBytes: self contentLength) asString.
	body := self toCr: crlfBody.
	self entityBody: body.
	self contentsStream nextPutAll: body

"	self entityBody: (aStream nextBytes: self contentLength) asString "! !

!HTTPPost methodsFor: 'reading' stamp: ''!
readFrom: aStream 
	| contentType |
	super readFrom: aStream.
	((self includesHeader: 'Content-Type')
		and: [self includesHeader: 'Content-Length'])
		ifFalse: [^self error: 'Bad POST header'].
	contentType := (self headerAt: 'Content-Type') value.
	contentType = 'application/x-www-form-urlencoded' ifTrue: [^self urlencodedDataFrom: aStream].
	contentType = 'multipart/form-data' ifTrue: [^self multipartDataFrom: aStream].
	self rawEntityBodyFrom: aStream! !

!HTTPPost methodsFor: 'private' stamp: 'lr 8/23/2003 18:28'!
urlencodedDataFrom: aStream 
	| e tokens |
	(self includesHeader: 'Content-Length') ifFalse: [^self].
	e := aStream nextBytes: self contentLength.
	tokens := e asString tokensBasedOn: $&.
	(tokens collect: [:each | each tokensBasedOn: $=]) do: 
			[:keyVal | 
			| datum |
			keyVal size = 1
				ifTrue: [ keyVal add: String new ].
			datum := HTTPPostDatum new.
			datum 
				value: (keyVal last 
						collect: [:char | char = $+ ifTrue: [Character space] ifFalse: [char]]) 
							decodedHTTP.
			self postDataAt: keyVal first put: datum]! !

HTTPRequest subclass: #HTTPPut
	instanceVariableNames: 'putData'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Swazoo-HTTP-Messages'!

!HTTPPut methodsFor: 'testing' stamp: ''!
isPut
	^true! !

!HTTPPut methodsFor: 'reading' stamp: ''!
octetDataFrom: aStream 
	(self includesHeader: 'Content-Length') 
		ifFalse: [^self error: 'Bad PUT header'].
	self putData: (aStream nextBytes: self contentLength)! !

!HTTPPut methodsFor: 'accessing' stamp: ''!
putData
	^putData! !

!HTTPPut methodsFor: 'private' stamp: ''!
putData: aString
	putData := aString! !

!HTTPPut methodsFor: 'reading' stamp: ''!
readFrom: aStream 
	| contentType |
	super readFrom: aStream.
	(self includesHeader: 'Content-Type') 
		ifFalse: [^self error: 'Bad PUT header'].
	contentType := (self headerAt: 'Content-Type') value.
	contentType = 'application/octet-stream' 
		ifTrue: [self octetDataFrom: aStream]
		ifFalse: [self urlencodedDataFrom: aStream]! !

!HTTPRequest class methodsFor: 'instance creation' stamp: 'mir 10/7/1999 16:59'!
for: aUrl in: aLoader
	^self new for: aUrl in: aLoader! !

!HTTPRequest class methodsFor: 'instance creation' stamp: ''!
newFor: aMethod 
	aMethod = 'GET' ifTrue: [^HTTPGet new].
	aMethod = 'HEAD' ifTrue: [^HTTPHead new].
	aMethod = 'POST' ifTrue: [^HTTPPost new].
	aMethod = 'PUT' ifTrue: [^HTTPPut new].
	MethodNotImplementedError raiseSignal! !

!HTTPRequest class methodsFor: 'instance creation' stamp: ''!
readFrom: aStream 
	| requestLine rs method inst ls |
	ls := LineStream on: aStream.
	requestLine := ls nextLine.
	[requestLine isEmpty]
		whileTrue: [requestLine := ls nextLine].
	rs := requestLine readStream.
	method := rs upTo: Character space.
	inst := self newFor: method.
	inst readFrom: aStream.
	inst parseURI: (rs upTo: Character space).
	inst parseHTTPVersion: (rs upTo: Character space).
	^inst! !

!HTTPRequest class methodsFor: 'instance creation' stamp: ''!
request: aUriString 
	^self 
		request: aUriString
		from: 'someHost'
		at: 'someIP'! !

!HTTPRequest class methodsFor: 'instance creation' stamp: ''!
request: aUriString from: aHostString at: anIPString 
	| request |
	request := self new.
	request addHeader: (GenericHeaderField name: 'Host' value: aHostString).
	request parseURI: aUriString.
	request ip: anIPString.
	^request! !

!HTTPRequest methodsFor: 'private-accessing' stamp: ''!
authenticated
	^authenticated! !

!HTTPRequest methodsFor: 'accessing-headers' stamp: ''!
connection
	^(self headerAt: 'Connection' ifAbsent: [^nil]) value! !

!HTTPRequest methodsFor: 'accessing-headers' stamp: ''!
contentLength
	^self headerValueOrNil: 'Content-Length'! !

!HTTPRequest methodsFor: 'accessing' stamp: ''!
contentsStream
	contentsStream ifNil: [contentsStream := ByteString new writeStream].
	^contentsStream! !

!HTTPRequest methodsFor: 'accessing-headers' stamp: ''!
cookie
	^self headerValueOrNil: 'COOKIE'! !

!HTTPRequest methodsFor: 'private-accessing' stamp: ''!
encrypted
	^encrypted! !

!HTTPRequest methodsFor: 'testing' stamp: ''!
hasCookie
	"check if  Cookie:  was in request header"

	^self includesHeader: 'Cookie'! !

!HTTPRequest methodsFor: 'accessing-headers' stamp: ''!
headerAt: aKey ifAbsent: aBlock
	^self headers at: aKey asUppercase ifAbsent: aBlock! !

!HTTPRequest methodsFor: 'accessing-headers' stamp: ''!
host
	^(self headerAt: 'Host' ifAbsent: [^String new]) value! !

!HTTPRequest methodsFor: 'accessing' stamp: ''!
httpVersion
	^httpVersion! !

!HTTPRequest methodsFor: 'private-accessing' stamp: ''!
httpVersion: anArray
	httpVersion := anArray! !

!HTTPRequest methodsFor: 'accessing-queries' stamp: ''!
includesQuery: aString 
	^self uri includesQuery: aString! !

!HTTPRequest methodsFor: 'private-initialize' stamp: ''!
initEnvironmentData
	environmentData := Dictionary new! !

!HTTPRequest methodsFor: 'private-initialize' stamp: 'lr 8/23/2003 16:04'!
initQueries
	self flag: #esug.
	"queries := Dictionary new"! !

!HTTPRequest methodsFor: 'private-initialize' stamp: ''!
initialize
	super initialize.
	self setTimestamp.
	self initQueries.
	self initEnvironmentData! !

!HTTPRequest methodsFor: 'accessing' stamp: ''!
ip
	^ip! !

!HTTPRequest methodsFor: 'private-accessing' stamp: ''!
ip: anObject
	ip := anObject! !

!HTTPRequest methodsFor: 'testing' stamp: ''!
isAuthenticated
	^self authenticated isNil not! !

!HTTPRequest methodsFor: 'testing' stamp: ''!
isClose
	| header |
	header := self connection.
	header isNil ifTrue: [^false].
	^'*close*' match: header! !

!HTTPRequest methodsFor: 'testing' stamp: ''!
isEncrypted
	^self encrypted isNil not! !

!HTTPRequest methodsFor: 'testing' stamp: ''!
isGet
	^false! !

!HTTPRequest methodsFor: 'testing' stamp: ''!
isHead
	^false! !

!HTTPRequest methodsFor: 'testing' stamp: ''!
isKeepAlive
	| header |
	header := self connection.
	header isNil ifTrue: [^false].
	^'*Keep-Alive*' match: self connection! !

!HTTPRequest methodsFor: 'testing' stamp: ''!
isPost
	^false! !

!HTTPRequest methodsFor: 'testing' stamp: ''!
isPut
	^false! !

!HTTPRequest methodsFor: 'private' stamp: ''!
linesFrom: aStream 
	| lines lineStream line |
	lines := OrderedCollection new.
	lineStream := LineStream on: aStream.
	
	[line := lineStream nextLine.
	line isEmpty] whileFalse: [lines add: line].
	^lines! !

!HTTPRequest methodsFor: 'private-initialize' stamp: ''!
parseHTTPVersion: aString 
	| rs major minor |
	rs := aString readStream.
	rs skipThrough: $/.
	major := (rs upTo: $.) asNumber.
	minor := rs upToEnd asNumber.
	self httpVersion: (Array with: major with: minor)! !

!HTTPRequest methodsFor: 'private-accessing' stamp: 'lr 8/23/2003 19:35'!
parseURI: aString 
	"| rs query uriStream |
	uriStream := WriteStream on: Array new.
	rs := aString readStream.
	(rs peekFor: $/) ifTrue: [uriStream nextPut: '/'].
	uriStream 
		nextPutAll: ((((rs upTo: $?) tokensBasedOn: $/) 
				reject: [:each | each isEmpty]) collect: [:each | each decodedHTTP]).
	self uri: uriStream contents.
	query := rs upToEnd.
	query isEmpty ifFalse: [self parseQueries: query]"

	"| rs |
	self uri: SwazooURI new.
	rs := ReadStream on: aString.
	'self uri protocol is for SSL set later !! '.
	self uri identifier: (rs upTo: $?).
	self uri queries: rs upToEnd.
	self uri host: self host"

	| rs queries attribute |
	self uri: SwazooURI new.
	rs := ReadStream on: aString.
	self uri identifier: (rs upTo: $?).
	self uri host: self host.

	queries := Dictionary new.
	(rs upToEnd tokensBasedOn: $&) do: [ :each |
		attribute := each tokensBasedOn: $=.
		attribute isEmpty ifFalse: [
			attribute size > 1 
				ifTrue: [ queries at: attribute first put: attribute second ]
				ifFalse: [ queries at: attribute first put: nil ] ] ].
	self uri queries: queries! !

!HTTPRequest methodsFor: 'accessing' stamp: ''!
peer
	^peer! !

!HTTPRequest methodsFor: 'private-accessing' stamp: ''!
peer: anObject
	peer := anObject! !

!HTTPRequest methodsFor: 'accessing-headers' stamp: ''!
port
	^self uri port! !

!HTTPRequest methodsFor: 'printing' stamp: 'lr 8/23/2003 19:47'!
printOn: aStream 
	aStream
		nextPutAll: 'an HTTPRequest'.
	self peer isNil ifFalse: [aStream nextPutAll: ' from: ';
		print: self peer ].
	aStream 
		nextPutAll: ' at: ';
		nextPutAll: self timestamp asString;
		nextPutAll: ' url: ';
		nextPutAll: self uri value! !

!HTTPRequest methodsFor: 'private-accessing' stamp: ''!
queries
	^self uri queries! !

!HTTPRequest methodsFor: 'accessing-queries' stamp: ''!
queryAt: aKey 
	^self uri queryAt: aKey! !

!HTTPRequest methodsFor: 'accessing-queries' stamp: ''!
queryAt: aKey ifAbsent: aBlock 
	^self uri queryAt: aKey ifAbsent: aBlock! !

!HTTPRequest methodsFor: 'private-initialize' stamp: ''!
readFrom: aStream 
	| lines |
	lines := self linesFrom: aStream.
	lines do: 
			[:each | 
			| field |
			self contentsStream
				nextPutAll: each;
				nextPut: Character cr.
			field := HeaderField fromLine: each.
			self headers at: field name put: field]! !

!HTTPRequest methodsFor: 'accessing-headers' stamp: ''!
referer
	^(self headerAt: 'Referer' ifAbsent: [^String new]) value! !

!HTTPRequest methodsFor: 'private-accessing' stamp: ''!
setAuthenticated 	
	authenticated := true! !

!HTTPRequest methodsFor: 'private-accessing' stamp: ''!
setEncrypted
	encrypted := true! !

!HTTPRequest methodsFor: 'private-initialize' stamp: 'avi 4/16/2004 16:20'!
setTimestamp
	timestamp := DateAndTime now! !

!HTTPRequest methodsFor: 'accessing' stamp: ''!
timestamp
	^timestamp! !

!HTTPRequest methodsFor: 'private' stamp: 'ab 8/20/2003 00:20'!
toCr: aString 
	^ aString withSqueakLineEndings ! !

!HTTPRequest methodsFor: 'private' stamp: 'ab 8/20/2003 00:20'!
toCrLf: aString 
	^ aString withInternetLineEndings ! !

!HTTPRequest methodsFor: 'accessing' stamp: ''!
uri
	^uri! !

!HTTPRequest methodsFor: 'private-accessing' stamp: ''!
uri: aSwazooURI
	uri := aSwazooURI! !

!HTTPRequest methodsFor: 'accessing' stamp: ''!
uriString
	^self uri identifier! !

!HTTPRequest methodsFor: 'accessing' stamp: ''!
urlString
	^self uri value! !

!HTTPRequest methodsFor: 'accessing-headers' stamp: ''!
userAgent
	^self headerValueOrNil: 'User-Agent'! !

!HTTPRequest methodsFor: 'testing' stamp: ''!
wantsConnectionClose
	self isClose ifTrue: [^true].
	^self httpVersion last = 0 and: [self isKeepAlive not]! !

HTTPMessage subclass: #HTTPResponse
	instanceVariableNames: 'code entity'
	classVariableNames: 'StatusCodes'
	poolDictionaries: ''
	category: 'Swazoo-HTTP-Messages'!

HTTPResponse subclass: #FileResponse
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Swazoo-HTTP-Messages'!

!FileResponse methodsFor: 'accessing-headers' stamp: ''!
contentType
	^self entity contentType! !

!FileResponse methodsFor: 'accessing' stamp: ''!
entity: aMimeObject 
	entity := aMimeObject! !

!FileResponse methodsFor: 'private-printing' stamp: ''!
printContentLengthOn: aStream 
	self entity isNil 
		ifFalse: 
			[aStream
				nextPutAll: 'Content-Length: ';
				print: self entity value fileSize.
			self crlfOn: aStream]! !

!FileResponse methodsFor: 'private-printing' stamp: ''!
printEntityOn: aStream 
	| rs |
	self entity isNil 
		ifFalse: 
			[rs := self entity value readStream.
			rs lineEndTransparent.
			
			[[[rs atEnd] whileFalse: [aStream nextPutAll: (rs nextAvailable: 2000)]] 
				ensure: [rs close]] 
					on: OsError
					do: [:ex | ex return]]! !

!FileResponse methodsFor: 'private-printing' stamp: ''!
printHeadersOn: aStream 
	self contentType: self entity contentType.
	super printHeadersOn: aStream! !

!HTTPResponse class methodsFor: 'response types' stamp: ''!
badRequest
	^self new code: 400! !

!HTTPResponse class methodsFor: 'response types' stamp: ''!
found
	^self new code: 302! !

!HTTPResponse class methodsFor: 'class initialization' stamp: ''!
initialize
	"self initialize"

	StatusCodes := (Dictionary new)
				add: 100 -> 'Continue';
				add: 101 -> 'Switching Protocols';
				add: 200 -> 'OK';
				add: 201 -> 'Created';
				add: 202 -> 'Accepted';
				add: 203 -> 'Non-Authoritative Information';
				add: 204 -> 'No Content';
				add: 205 -> 'Reset Content';
				add: 206 -> 'Partial Content';
				add: 300 -> 'Multiple Choices';
				add: 301 -> 'Moved Permanently';
				add: 302 -> 'Found';
				add: 303 -> 'See Other';
				add: 304 -> 'Not Modified';
				add: 305 -> 'Use Proxy';
				add: 307 -> 'Temporary Redirect';
				add: 400 -> 'Bad Request';
				add: 401 -> 'Unauthorized';
				add: 402 -> 'Payment Required';
				add: 403 -> 'Forbidden';
				add: 404 -> 'Not Found';
				add: 405 -> 'Method Not Allowed';
				add: 406 -> 'Not Acceptable';
				add: 407 -> 'Proxy Authentication Required';
				add: 408 -> 'Request Time-out';
				add: 409 -> 'Conflict';
				add: 410 -> 'Gone';
				add: 411 -> 'Length Required';
				add: 412 -> 'Precondition Failed';
				add: 413 -> 'Request Entity Too Large';
				add: 414 -> 'Request-URI Too Large';
				add: 415 -> 'Unsupported Media Type';
				add: 416 -> 'Requested range not satisfiable';
				add: 417 -> 'Expectation Failed';
				add: 500 -> 'Internal Server Error';
				add: 501 -> 'Not Implemented';
				add: 502 -> 'Bad Gateway';
				add: 503 -> 'Service Unavailable';
				add: 504 -> 'Gateway Time-out';
				add: 505 -> 'HTTP Version not supported';
				yourself! !

!HTTPResponse class methodsFor: 'response types' stamp: ''!
internalServerError
	^(self new)
		code: 500;
		entity: '<HTML>
<HEAD><TITLE>Not Found</TITLE></HEAD>
<BODY>
<H1>500 Internal Server Error</H1>
<P>The server experienced an error while processing this request.  If this problem persists, please contact the webmaster.</P>
</BODY></HTML>'! !

!HTTPResponse class methodsFor: 'response types' stamp: ''!
movedPermanently
	^self new code: 301! !

!HTTPResponse class methodsFor: 'response types' stamp: ''!
notFound
	^(self new)
		code: 404;
		entity: '<HTML>
<HEAD><TITLE>Not Found</TITLE></HEAD>
<BODY>
<H1>404 Not Found</H1>
<P>The requested resource was not found on this server.</P>
</BODY></HTML>'! !

!HTTPResponse class methodsFor: 'response types' stamp: ''!
notImplemented
	^self new code: 501! !

!HTTPResponse class methodsFor: 'response types' stamp: ''!
notModified
	^self new code: 304! !

!HTTPResponse class methodsFor: 'response types' stamp: ''!
ok
	^self new code: 200! !

!HTTPResponse class methodsFor: 'response types' stamp: ''!
redirectLink
	^self new code: 302! !

!HTTPResponse class methodsFor: 'response types' stamp: ''!
seeOther
	^self new code: 303! !

!HTTPResponse methodsFor: 'private-initialize' stamp: 'avi 4/16/2004 16:20'!
addDateHeader
	"Should really get current UTC, not current local time"

	#FIXME.
	self addHeaderName: 'Date' value: DateAndTime now rfc1123String! !

!HTTPResponse methodsFor: 'private-initialize' stamp: 'avi 4/16/2004 16:17'!
addServerHeader
	^self 
		addHeader: (GenericHeaderField name: 'Server' value: self serverVersion)! !

!HTTPResponse methodsFor: 'accessing' stamp: ''!
code
	^code! !

!HTTPResponse methodsFor: 'private-initialize' stamp: ''!
code: anInteger 
	code := anInteger! !

!HTTPResponse methodsFor: 'accessing-headers' stamp: ''!
contentType
	^(self headerAt: 'Content-Type' ifAbsent: [^'application/octet-stream']) 
		value! !

!HTTPResponse methodsFor: 'accessing-headers' stamp: ''!
contentType: aString 
	self addHeaderName: 'Content-Type' value: aString! !

!HTTPResponse methodsFor: 'accessing-headers' stamp: ''!
cookie: aString 
	self addHeaderName: 'Set-Cookie' value: aString! !

!HTTPResponse methodsFor: 'private-printing' stamp: ''!
crlfOn: aStream 
	aStream
		nextPut: Character cr;
		nextPut: Character lf! !

!HTTPResponse methodsFor: 'private-printing' stamp: ''!
endHeaderOn: aStream 
	self crlfOn: aStream! !

!HTTPResponse methodsFor: 'accessing' stamp: ''!
entity
	^entity! !

!HTTPResponse methodsFor: 'accessing' stamp: ''!
entity: anEntity 
	entity := anEntity asByteArray! !

!HTTPResponse methodsFor: 'private' stamp: ''!
informConnectionClose
	self addHeaderName: 'Connection' value: 'close'! !

!HTTPResponse methodsFor: 'private-initialize' stamp: ''!
initialize
	super initialize.
	self addServerHeader.
	self addDateHeader! !

!HTTPResponse methodsFor: 'testing' stamp: ''!
isBadRequest
	^self code = 400! !

!HTTPResponse methodsFor: 'testing' stamp: ''!
isFound
	^self code = 302! !

!HTTPResponse methodsFor: 'testing' stamp: ''!
isInternalServerError
	^self code = 500! !

!HTTPResponse methodsFor: 'testing' stamp: ''!
isMovedPermanently
	^self code = 301! !

!HTTPResponse methodsFor: 'testing' stamp: ''!
isNotFound
	^self code = 404! !

!HTTPResponse methodsFor: 'testing' stamp: ''!
isNotImplemented
	^self code = 501! !

!HTTPResponse methodsFor: 'testing' stamp: ''!
isNotModified
	^self code = 304! !

!HTTPResponse methodsFor: 'testing' stamp: ''!
isOk
	^self code = 200! !

!HTTPResponse methodsFor: 'testing' stamp: ''!
isRedirectLink
	^self code = 302! !

!HTTPResponse methodsFor: 'testing' stamp: ''!
isSeeOther
	^self code = 303! !

!HTTPResponse methodsFor: 'accessing-headers' stamp: ''!
location: aString 
	self addHeaderName: 'Location' value: aString! !

!HTTPResponse methodsFor: 'private-printing' stamp: ''!
printContentLengthOn: aStream 
	self entity isNil 
		ifFalse: 
			[aStream
				nextPutAll: 'Content-Length: ';
				print: self entity size.
			self crlfOn: aStream]! !

!HTTPResponse methodsFor: 'private-printing' stamp: ''!
printEntityOn: aStream 
	self entity isNil ifFalse: [aStream nextPutBytes: self entity]! !

!HTTPResponse methodsFor: 'private-printing' stamp: ''!
printHeadersOn: aStream 
	self headers keysAndValuesDo: 
			[:key :val | 
			aStream
				nextPutAll: key asString;
				nextPutAll: ': ';
				nextPutAll: val value.
			self crlfOn: aStream]! !

!HTTPResponse methodsFor: 'private-printing' stamp: ''!
printStatusOn: aStream 
	aStream
		nextPutAll: 'HTTP/1.1 ';
		print: self code;
		space;
		nextPutAll: (StatusCodes at: self code).
	self crlfOn: aStream! !

!HTTPResponse methodsFor: 'private' stamp: 'avi 4/16/2004 16:19'!
serverVersion
	^ 'Swazoo-HTTP on ', (SystemVersion current asString)
	! !

!HTTPResponse methodsFor: 'accessing' stamp: ''!
writeHeaderTo: aStream 
	self printStatusOn: aStream.
	self printHeadersOn: aStream.
	self printContentLengthOn: aStream.
	self endHeaderOn: aStream! !

!HTTPResponse methodsFor: 'accessing' stamp: ''!
writeTo: aStream 
	self writeHeaderTo: aStream.
	self printEntityOn: aStream! !

Object subclass: #HeaderField
	instanceVariableNames: 'value'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Swazoo-HTTP-Headers'!

HeaderField subclass: #GenericHeaderField
	instanceVariableNames: 'name'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Swazoo-HTTP-Headers'!

!GenericHeaderField class methodsFor: 'instance creation' stamp: ''!
name: aFieldString value: aValueString 
	^(self new)
		name: aFieldString;
		value: aValueString! !

!GenericHeaderField methodsFor: 'accessing' stamp: ''!
combine: aGenericHeaderField 
	self value: self value , ',' , aGenericHeaderField value! !

!GenericHeaderField methodsFor: 'accessing' stamp: ''!
name
	^name! !

!GenericHeaderField methodsFor: 'private-initialize' stamp: ''!
name: aString 
	name := aString! !

!GenericHeaderField methodsFor: 'accessing' stamp: ''!
values
	^(self value tokensBasedOn: $,) collect: [:each | each trimBlanks]! !

!HeaderField class methodsFor: 'instance creation' stamp: ''!
fromLine: aString 
	| rs headerName headerValue inst fieldClass |
	rs := aString readStream.
	headerName := (rs upTo: $:) trimBlanks asUppercase.
	headerValue := rs upToEnd trimBlanks.
	fieldClass := SpecificHeaderField allSubclasses 
				detect: [:each | each fieldName = headerName]
				ifNone: [nil].
	inst := fieldClass isNil 
				ifTrue: [GenericHeaderField new name: headerName]
				ifFalse: [fieldClass new].
	inst valueFrom: headerValue.
	^inst! !

!HeaderField class methodsFor: 'instance creation' stamp: ''!
new
	^super new initialize! !

!HeaderField methodsFor: 'accessing' stamp: ''!
combine: aHeaderField 
	^self subclassResponsibility! !

!HeaderField methodsFor: 'private-initialize' stamp: ''!
initialize! !

!HeaderField methodsFor: 'testing' stamp: ''!
isContentDisposition
	^false! !

!HeaderField methodsFor: 'testing' stamp: ''!
isContentType
	^false! !

!HeaderField methodsFor: 'accessing' stamp: ''!
name
	^self subclassResponsibility! !

!HeaderField methodsFor: 'accessing' stamp: ''!
value
	^value! !

!HeaderField methodsFor: 'accessing' stamp: ''!
value: aString
	value := aString! !

!HeaderField methodsFor: 'private-initialize' stamp: ''!
valueFrom: aString
	self value: aString! !

!HeaderField methodsFor: 'accessing' stamp: ''!
values
	^self subclassResponsibility! !

HeaderField subclass: #SpecificHeaderField
	instanceVariableNames: 'parameters'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Swazoo-HTTP-Headers'!

SpecificHeaderField subclass: #ContentDispositionField
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Swazoo-HTTP-Headers'!

!ContentDispositionField class methodsFor: 'accessing' stamp: ''!
fieldName
	^'CONTENT-DISPOSITION'! !

!ContentDispositionField methodsFor: 'testing' stamp: ''!
isContentDisposition
	^true! !

!ContentDispositionField methodsFor: 'private-initialize' stamp: ''!
valueFrom: aString 
	| rs |
	rs := aString readStream.
	self value: (rs upTo: $;) trimBlanks.
	self readParametersFrom: rs! !

SpecificHeaderField subclass: #ContentLengthHeader
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Swazoo-HTTP-Headers'!

!ContentLengthHeader classSide methodsFor: 'accessing' stamp: ''!
fieldName
	^'CONTENT-LENGTH'! !

!ContentLengthHeader methodsFor: 'private-initialize' stamp: ''!
valueFrom: aString 
	self value: aString asNumber! !

SpecificHeaderField subclass: #ContentTypeField
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Swazoo-HTTP-Headers'!

!ContentTypeField class methodsFor: 'accessing' stamp: ''!
fieldName
	^'CONTENT-TYPE'! !

!ContentTypeField methodsFor: 'testing' stamp: ''!
isContentType
	^true! !

!ContentTypeField methodsFor: 'private-initialize' stamp: ''!
valueFrom: aString 
	| rs |
	rs := aString readStream.
	self value: (rs upTo: $;) trimBlanks.
	self readParametersFrom: rs! !

!SpecificHeaderField class methodsFor: 'accessing' stamp: ''!
fieldName
	^self subclassResponsibility! !

!SpecificHeaderField methodsFor: 'accessing' stamp: ''!
combine: aGenericHeaderField 
	self error: 'Not supported'! !

!SpecificHeaderField methodsFor: 'private-initialize' stamp: ''!
initParameters
	parameters := Dictionary new! !

!SpecificHeaderField methodsFor: 'private-initialize' stamp: ''!
initialize
	super initialize.
	self initParameters! !

!SpecificHeaderField methodsFor: 'accessing' stamp: ''!
name
	^self class fieldName! !

!SpecificHeaderField methodsFor: 'accessing' stamp: ''!
parameterAt: aString 
	^self parameterAt: aString ifAbsent: [nil]! !

!SpecificHeaderField methodsFor: 'accessing' stamp: ''!
parameterAt: aString ifAbsent: aBlock 
	^self parameters at: aString ifAbsent: aBlock! !

!SpecificHeaderField methodsFor: 'private-utility' stamp: ''!
parameters
	parameters isNil ifTrue: [self initParameters].
	^parameters! !

!SpecificHeaderField methodsFor: 'private-utility' stamp: ''!
readParametersFrom: rs 
	[rs atEnd] whileFalse: 
			[| paramString tok |
			paramString := rs upTo: $;.
			tok := (paramString tokensBasedOn: $=) collect: [:each | each trimBlanks].
			self parameters at: tok first put: tok last]! !

!SpecificHeaderField methodsFor: 'accessing' stamp: ''!
values
	^Array with: self value! !

Object subclass: #LineStream
	instanceVariableNames: 'stream'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Swazoo-HTTP-Support'!

!LineStream classSide methodsFor: 'instance creation' stamp: ''!
on: aStream 
	^self new stream: aStream! !

!LineStream methodsFor: 'accessing' stamp: ''!
nextLine
	| ws |
	ws := WriteStream on: String new.
	ws nextPutAll: (self stream upTo: Character cr).
	^self stream peek = Character lf 
		ifTrue: 
			[self stream next.
			ws contents]
		ifFalse: [self error: 'CR without LF']! !

!LineStream methodsFor: 'private-initialize' stamp: ''!
stream
	^stream! !

!LineStream methodsFor: 'private-initialize' stamp: ''!
stream: aStream
	stream := aStream! !

Object subclass: #MimeObject
	instanceVariableNames: 'contentType value'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Swazoo-HTTP-Support'!

MimeObject subclass: #HTTPPostDatum
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Swazoo-HTTP-Support'!

!HTTPPostDatum methodsFor: 'private-accessing' stamp: ''!
defaultContentType
	^'text/plain'! !

!MimeObject methodsFor: 'accessing' stamp: ''!
contentType
	^contentType isNil ifTrue: [self defaultContentType] ifFalse: [contentType]! !

!MimeObject methodsFor: 'accessing' stamp: ''!
contentType: anObject
	contentType := anObject! !

!MimeObject methodsFor: 'private-accessing' stamp: ''!
defaultContentType
	^'application/octet-stream'! !

!MimeObject methodsFor: 'accessing' stamp: ''!
value
	^value! !

!MimeObject methodsFor: 'accessing' stamp: ''!
value: anObject
	value := anObject! !

Object subclass: #SwazooURI
	instanceVariableNames: 'protocol hostname port identifier queries'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Swazoo-HTTP-Support'!

!SwazooURI class methodsFor: 'instance creation' stamp: ''!
new
	^super new initialize! !

!SwazooURI class methodsFor: 'instance creation' stamp: ''!
value: aString
	^self new value: aString! !

!SwazooURI methodsFor: 'private-accessing' stamp: ''!
defaultPort
	^80! !

!SwazooURI methodsFor: 'accessing' stamp: ''!
host
	| ws |
	ws := WriteStream on: String new.
	ws nextPutAll: self hostname.
	self port = self defaultPort 
		ifFalse: 
			[ws
				nextPut: $:;
				print: self port].
	^ws contents! !

!SwazooURI methodsFor: 'accessing' stamp: ''!
host: aString 
	| rs |
	rs := ReadStream on: aString.
	self hostname: (rs upTo: $: ).
	rs atEnd ifFalse: [self port: rs upToEnd asNumber]! !

!SwazooURI methodsFor: 'accessing' stamp: ''!
hostname
	^hostname! !

!SwazooURI methodsFor: 'accessing' stamp: ''!
hostname: aHostname 
	hostname := aHostname! !

!SwazooURI methodsFor: 'accessing' stamp: ''!
identifier
	^identifier! !

!SwazooURI methodsFor: 'accessing' stamp: ''!
identifier: anObject
	identifier := anObject! !

!SwazooURI methodsFor: 'accessing' stamp: ''!
identifierPath
	| parts |
	parts := (self identifier tokensBasedOn: $/) 
				collect: [:each | each decodedHTTP].
	self identifier first = $/ ifTrue: [parts addFirst: '/'].
	^parts reject: [:each | each isEmpty]! !

!SwazooURI methodsFor: 'accessing-queries' stamp: ''!
includesQuery: aString 
	^self queryData includesKey: aString! !

!SwazooURI methodsFor: 'private-initialize' stamp: ''!
initialize
	self port: self defaultPort! !

!SwazooURI methodsFor: 'testing' stamp: ''!
isDirectory
	^self identifier last = $/! !

!SwazooURI methodsFor: 'accessing' stamp: ''!
port
	^port! !

!SwazooURI methodsFor: 'accessing' stamp: ''!
port: anInteger 
	port := anInteger! !

!SwazooURI methodsFor: 'accessing' stamp: ''!
protocol
	protocol isNil ifTrue: [self protocol: 'http'].
	^protocol! !

!SwazooURI methodsFor: 'accessing' stamp: ''!
protocol: aString
	protocol := aString.! !

!SwazooURI methodsFor: 'accessing-queries' stamp: ''!
queries
	^queries! !

!SwazooURI methodsFor: 'accessing-queries' stamp: ''!
queries: anObject
	queries := anObject! !

!SwazooURI methodsFor: 'accessing-queries' stamp: ''!
queryAt: aString 
	^self queryAt: aString ifAbsent: [nil]! !

!SwazooURI methodsFor: 'accessing-queries' stamp: ''!
queryAt: aString ifAbsent: aBlock 
	^self queryData at: aString ifAbsent: aBlock! !

!SwazooURI methodsFor: 'accessing-queries' stamp: 'lr 8/25/2003 15:31'!
queryData
	| dict |
	" #sw. This was parsing ?ABC the same as ?ABC=ABC"
	self flag: #esug.
	"dict := Dictionary new.
	(self queries tokensBasedOn: $&) do: 
			[:keyVal | 
			| tok |
			tok := keyVal tokensBasedOn: $=.
			dict 
				at: tok first 
				put: (tok size = 1
					ifTrue: [nil]
					ifFalse: [tok last decodedHTTP])].
	^dict"

	^queries isString
		ifTrue: [ 
			dict := Dictionary new.
			(self queries tokensBasedOn: $&) do: 
					[:keyVal | 
					| tok |
					tok := keyVal tokensBasedOn: $=.
					dict 
						at: tok first 
						put: (tok size = 1
							ifTrue: [nil]
							ifFalse: [tok last decodedHTTP])].
			dict ]
		ifFalse: [ queries ]! !

!SwazooURI methodsFor: 'accessing' stamp: ''!
value
	| ws |
	ws := WriteStream on: String new.
	ws nextPutAll: self protocol, '://'.
	ws nextPutAll: self host.
	ws nextPutAll: self identifier.
	(self queries isNil or: [self queries isEmpty]) 
		ifFalse: 
			[ws
				nextPut: $?;
				nextPutAll: self queries].
	^ws contents! !

!Stream methodsFor: '*swazoo-http' stamp: 'avi 4/16/2004 16:16'!
nextBytes: aNumber
	^ (self next: aNumber) asByteArray! !

Error subclass: #MethodNotImplementedError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Swazoo-HTTP-Support'!

!String methodsFor: '*swazoo-http' stamp: ''!
decodedHTTP
	| ws rs |
	ws := self class new writeStream.
	rs := self readStream.
	[rs atEnd]
		whileFalse: 
			[| char |
			char := rs next.
			char = $%
				ifTrue: [ws nextPut: ('16r' , (rs next: 2)) asNumber asCharacter]
				ifFalse: [ws nextPut: char]].
	^ws contents! !

!String methodsFor: '*swazoo-http' stamp: ''!
encodedHTTP
	| ws |
	ws := self class new writeStream.
	self do: 
			[:char | 
			char isHTTPReserved 
				ifTrue: 
					[ws nextPut: $%.
					char asInteger 
						printOn: ws
						paddedWith: $0
						to: 2
						base: 16]
				ifFalse: [ws nextPut: char]].
	^ws contents! !

!String methodsFor: '*swazoo-http' stamp: 'ab 8/19/2003 01:21'!
tokensBasedOn: aCharacter
	^ self findTokens: aCharacter asString! !

!String methodsFor: '*swazoo-http' stamp: ''!
trimBlanks
	| first last |
	self isEmpty ifTrue: [^self].
	first := 1.
	last := self size.
	[(self at: first) isSeparator and: [first < last]] 
		whileTrue: [first := first + 1].
	[(self at: last) isSeparator and: [last >= first]] 
		whileTrue: [last := last - 1].
	^self copyFrom: first to: last! !
HTTPResponse initialize!
